' Beschreibung:
' Programm: Liest zwei ADC-  bzw. einen Digitaleingang aus und gibt die Werte als Morse- Text
' per NF-Ton und parallel als Keyer- Schaltsignal aus.
' Der ADC ist mit Spannungsteilern 1:9 (82 kOhm und 10 kOhm) beschaltet; die Referenz des ADC
' ist die intern erzeugte  1,1V- Spannung. Mebereich also 0....10 V
' Die Pausenzeit zwischen den Aussendungen wird duch die Konstante "WDzaehler" eingestellt (Vielfache
' von 8 sec)
' PB1 lst eine Alarmmeldung aus, wenn er auf Masse gezogen wird.
'
'
' Sprache: Bascom-AVR-Basic
'
' Erluterungen:
'
' Codierung der Morsezeichen: 1 Byte pro Zeichen, niederwertiges Bit zuerst.
' Ist das Bit 0: Morsepunkt (dit)
' Ist das Bit 1: Moersestrich (dah)
' Da Morsezeichen unterschiedlich lang sind, ist das Ende- Zeichen ebenfalls ein 1-Bit.
'
' Beispiel Morsezeichen "a" (dit dah): &B00000110
' Beispiel Morsezeichen "9" (dah dah dah dah dit): &B00101111
'
' Auslesen der codierten Zeichen:
' niederwertigstes Bit auslesen, Byte um 1 Stelle nach rechts schieben, niederwertigstes Bit
' auslesen. Ist der Gesamtwert des Bytes 1 (Ende-marke), ist das Zeichen komplett.
'
'
' Timer0 erzeugt den Mithrton. Pin PB0 wird als Timer0-Ausgang OC0A verwendet.
' Pin PB3 ist Keyer-Ausgang.
'
' Pin PB1 ist der Alarmeingang; wird im Ruhezustand ber internen Pullup auf " High" gezogen
' Pin PB2 und PB4 sind die Analogeingnge. Sie sind mit Spannungsteilern 1:9 (82 kOhm und 10 kOhm)
' beschaltet; die Referenz des ADC ist die intern erzeugte  1,1V- Spannung. Mebereich also 0....10 V

' Der ACD kann als Spannungsreferenz wahlweise die Betriebsspannung nutzen oder interne 1,1 Volt.
' Will man ein Poti abfragen, whlt man Betriebsspannung ("AVCC"), will man (ber einen Spannungs-
' teiler) eine externe Spannung messen, die 1,1 Volt ("Internal").
'
'
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' Grundeinstelllungen:
$regfile = "ATtiny13.dat"
$crystal = 1200000                                          ' Standard- Takt
$hwstack = 8                                                ' nur max. 4 Unterprogramme, spart SRAM
$swstack = 0                                                ' wird nicht bentigt, spart SRAM
$framesize = 0                                              ' dito

'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' Konstanten:
Const Dit = 60                                              ' Lnge eines Punkts, daraus ergibt sich die Morsegeschwindigkeit
Const Dah = Dit * 3                                         ' Lnge eines Strichs
Const Wort = Dit * 8                                        ' Lnge der Pause zwischen Wrtern
Const Tonhoehe = 14                                         ' Tonhhe
Const Wdzaehler = 4                                         ' bestimmt Pausenzeit zwischen den Sendungen;


'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' Variablen:

Dim I As Byte                                               ' wird in Zhlschleife verwendet
Dim J As Byte
Dim N As Byte                                               ' Aktueller Morsebuchstabe
Dim M As Byte                                               ' wird frs Auslesen der Morsebits verwendet
Dim L As Byte                                               ' niederwertigstes Bit von M verwendet
Dim Ana As Word                                             ' ADC
Dim Zehner As Word                                          ' Zehnerstelle des umgerechneten Analogwertes
Dim Einer As Word                                           ' Einerstelle des umgerechneten Analogwertes
Dim E As Byte                                               ' Multiplikator, um Watchdog- Timer zu verlngern


'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' Initialisierung:
'
Acsr.acd = 1                                               ' Analog-Komparator ausschalten, spart etwas Strom
Ddrb = &B00001001                                          ' Pins PB3+PB0 als Ausgnge konfigurieren (1), Rest ist Eingang (0)
Portb = &B00000010                                          ' PullUp-Widerstand fr Pin PB1 aktivieren
Didr0 = &B00011101                                          ' digitale Eingnge PinB.0 und PinB.2 - PinB.4 deaktivieren
'
'
' Morseton mit Timer0 erzeugen:
'
' Timer0 zhlt das Register TCNT0 hoch. Es wird mit Register OCR0A verglichen.
' Wenn TCNT0 = OCR0A, wird Output OC0A/PB0 umgeschaltet (getoggled)
' Timer0 erzeugt so Dauerton; er wird jedoch nur an den Ausgangspin durchgeschaltet,
' wenn das Datenrichtungs- Register DDRB.0 = 1 ist. Bei 0 wird der Ton unterdrckt.
'
Config Timer0 = Counter , Prescale = 64 , Compare A = Toggle , Clear Timer = 1
Ocr0a = Tonhoehe


'-------------------------------------------------------------------------------
' Hauptprogramm:


    Ddrb.0 = 0                                              ' Ton und Keyer aus
    Portb.3 = 0
    E = Wdzaehler + 1

 Do
  E = E + 1                                                 ' Watchdog- Verlngerer inkrementieren
   If E > Wdzaehler Then                                    ' Watchdog- Verlngerer; Bedingung erfllt
       Stop Watchdog

        Restore Message3
        For N = 1 To 254                                    'wir senden Message 3
           Read M
           If M = 255 Then                                  ' Ende- Zeichen
             Exit For
           End If
           Gosub Morse
        Next

        Restore Message2                                    ' Zeiger auf Anfang der Message 2 setzen
        For J = 1 To 2                                      ' Zhlschleife fr 2 ADC- Eingnge
           For N = 11 To 15                                 ' wir senden " adc1" bzw " adc2"
             Read M
             Gosub Morse
           Next

           Gosub Messen                                     ' wir fragen den AD- Wandler ab
           N = Zehner                                       ' Zehnerstelle des Messwerts
           Gosub Nachschlagen                               ' Morsezeichen ermitteln und aussenden

           Read M                                           ' wir senden den Dezimaltrenner (Punkt)
           Gosub Morse

           N = Einer                                        ' Einerstelle  des Messwerts
           Gosub Nachschlagen                               ' Morsezeichen ermitteln und aussenden

           Read M                                           ' wir senden das "v"
           Gosub Morse
        Next J

        Wait 1
        If Pinb.1 = 0 Then
            For N = 1 To 5                                  ' wir senden "alarm"
               Read M
               Gosub Morse
            Next
        End If
    E = 0                                                   ' Watchdog- Verlngerer zurcksetzen
   End If

      Sreg.7 = 0                                            ' Interrupts sperren
      Wdtcr = &B00111001                                    ' Schreiben auf WDTCR Vorbereiten
      Mcusr.3 = 1                                           ' Schreiben auf WDTCR vorbereiten
      Wdtcr = &B01100001                                    ' Interrupt statt Reset, Watchdog- Timer auf 8 sec setzen
      Sreg.7 = 1                                            ' Interrupts wieder freigeben
      Powerdown                                             'maximale Stromersparnis

 Loop


 ' Auslesen des ADC

   Messen:
   Config Adc = Single , Prescaler = Auto , Reference = Internal       'Refrenz: 1,1 V
   Start Adc
   Waitms Wort                                              ' Wortlnge warten, bis sich alles eingeschwungen hat
      Ana = Getadc(j)
      Ana = Ana / 10
      Zehner = Ana / 10                                     ' Zerlegung in Einzelzahlen
      Einer = Ana Mod 10
   Stop Adc                                                 ' Strom sparen
   Return



' Morsezeichenerzeugung:

Nachschlagen:

       M = Lookup(n , Message1)

Morse:

     If M = 0 Then                                          ' Sonderfall: Leerzeichen;
        Waitms Wort
        Goto Sign_end
     End If

     For I = 1 To 8
       If M = 1 Then                                        ' Das Byte hat nur noch den Wert 1; Zeichenende!
          Goto Sign_end
       End If
        L = M And &B0000001                                 ' niederwertigstes Bit lesen
         Ddrb.0 = 1                                         ' Ausgnge einschalten
         Portb.3 = 1
       If L = 1 Then
          Waitms Dah                                        ' ist das Bit 1 -> dah
         Else
          Waitms Dit                                        ' ist das Bit 0 -> dit
       End If
         Ddrb.0 = 0                                         ' Ausgnge abschalten
         Portb.3 = 0
       Waitms Dit                                           ' Pause innerhalb des Morsezeichens
       Shift M , Right , 1                                  ' Bits um eine Stelle nach rechts shiften
     Next I
     Sign_end:
     Waitms Dah                                             ' Pause zwischen Morsezeichen
   Return
End

'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' Codierung der Morsezeichen. Bei einem Prozessor mit viel SRAM wrde man alle Zeichen
' in ein Array einlesen (bersetzungstabelle), den eigentlichen Bakentext in einen String packen
' und diesen String zeichenenweise abarbeiten.
'
' Da der Attiny13 nur ca. 64 Byte SRAM hat, waere nur Platz fr etwa 50 Bytes. Daher ist der Weg hier
' anders:  Der Bakentext ist (in Morsecodierung) in Datazeilen (also im Programmspeicher) abgelegt,
' allerdings durch Verwendung von Konstanten in einer leicht lesbaren Schreibweise.
' Die Data-Zeilen werden sequenziell ausgelesen.
' Nur die wirklich bentigten Zeichen werden (beim Kompilieren des Programmes) umgewandelt.


Const #a = &B00000110
Const #b = &B00010001
Const #c = &B00010101
Const #d = &B00001001
Const #e = &B00000010
Const #f = &B00010100
Const #g = &B00001011
Const #h = &B00010000
Const #i = &B00000100
Const #j = &B00011110
Const #k = &B00001101
Const #l = &B00010010
Const #m = &B00000111
Const #n = &B00000101
Const #o = &B00001111
Const #p = &B00010110
Const #q = &B00011011
Const #r = &B00001010
Const #s = &B00001000
Const #t = &B00000011
Const #u = &B00001100
Const #v = &B00011000
Const #w = &B00001110
Const #x = &B00011001
Const #y = &B00011101
Const #z = &B00010011
Const #ae = &B00011010
Const #oe = &B00010111
Const #ue = &B00011100
Const #0 = &B00111111 :
Const #1 = &B00111110:
Const #2 = &B00111100 :
Const #3 = &B00111000 :
Const #4 = &B00110000 :
Const #5 = &B00100000 :
Const #6 = &B00100001 :
Const #7 = &B00100011 :
Const #8 = &B00100111 :
Const #9 = &B00101111 :

Const Leerz = 0
Const Anfuehrz = &B01010010
Const Kl_auf = &B00101101
Const Kl_zu = &B01101101
Const Plus = &B00101010
Const Komma = &B01110011
Const Minus = &B01100001
Const Punkt = &B01101010
Const Slash = &B00101001
Const Dopp_pkt = &B01000111
Const Gleich = &B00110001
Const Fragez = &B01001100
Const Ende = 255


Message1:

Data #0 , #1 , #2 , #3 , #4 , #5 , #6 , #7 , #8 , #9 , #ue
'#ue: berlauf (tritt auf, wenn Ana > 1000)

Message2:

Data Leerz , #a , #d , #c , #1 , Punkt , #v
Data Leerz , #a , #d , #c , #2 , Punkt , #v
Data #a , #l , #a , #r , #m,




Message3:

Data #d , #k , #5 , #b , #u , Leerz , #b , #e , #a , #c , #o , #n , Leerz , Leerz
Data #q , #t , #h , Leerz , #b , #r , #e , #m , #e , #n , Leerz , Leerz
Data #p , #w , #r , Leerz , #1 , #0 , #0 , #m , #w , Leerz
Data #a , #n , #t , Leerz , #d , #i , #p , #o , #l , #e , Leerz , #u , #p , #7 , #m

Data Ende                                                   ' nicht vergessen !